

import ij.gui.PlotWindow;
import ij.gui.GenericDialog;
import java.awt.Color;
import java.text.DecimalFormat;
import ij.IJ;
import ij.ImagePlus;
import ij.process.ImageProcessor;
import ij.plugin.*;

/**Plugin for ImageJ returns an ImageJ results window showing
 * fractal dimensions, lacunarity,
 * and other measures for binary images.
 *<p>
 * Operates on single or multiple image files.
 * Popups ask the user for parameters.  Shows regression lines for the box
 * counting dimension at the original location, an average box counting
 * dimension, a minimum cover dimension (from the minimum count at each
 * box size over multiple origins), and a slope corrected version.
 * </p><p>
 * The user can choose to
 * use one series of grids, where each smaller sized grid is
 * always positioned at the top left corner of the image.  Or the user
 * can choose to use several such series, where
 * each smaller size within a series always starts at the same position
 * but the absolute position of the series is randomly chosen from
 * between the top left corner and the edge of the screen
 * (a border is added to images to ensure there is
 * space to do this). 
 *</p><p>
 * If the subareas option is selected, calculates a distribution of
 * local fractal dimensions over
 * smaller subareas of the image then delivers a colour-coded graphic as well
 * as text values of these local dimensions.  This function only works on
 * single images and the option to open multiple images is not allowed in this
 * mode.  Alternatively, contact the author for an algorithm to calculate data
 * for the multifractal spectrum of entropies; this is a great deal of
 * data that clutters the results file so is not automatically included.
 *</p><p>
 * Delivers information about variability in the image.  Calculates
 * the coefficient of variation in the fractal dimension over multiple
 * locations and two lacunarity values,
 * the average coefficient of variation in the pixel distribution
 * and the probability density lacunarity.
 *</p><p>
 * Calculates the horizontal
 * and vertical axes of the rectangle and oval enclosing the image,
 * and the perimeter and area of the convex hull enclosing the image.
 * Delivers circularity based on the convex hull.
 *
 * This program is free software distributed in the same way that ImageJ is.
 *
 * A. Karperien, Charles Sturt University, August 2003
 * Copyright (C) 2003 Audrey Karperien
 * For futher information or to send comments, bugs, and feature requests
 * contact{@link <a href="mailto:akarpe@postoffice.csu.edu.au"> the author</a>}
 * or see
 *{@link <a href="http://www.geocities.com/akarpe@sbcglobal.net/box.html">
 * FracLac page</a>}
 * .<p>file: FracLac_2003August__1.java</p>
 * <p>IDE: Sun ONE Studio 4, Community Edition</p>
 * @version FracLac2003: Version 2.4 August 31, 2003
 * @author Audrey Karperien (akarpe01@postoffice.csu.edu.au)
 * @since J2SKD1.4.1_02b
 */

public class FracLac_2003AugustC implements ij.plugin.PlugIn
{   ///////////////////////////VARIABLES//////////////////////////////
    
    //////////////////////////////////////////////////////////////////
    /////////////////////     working variables    ///////////////////
    //////////////////////////////////////////////////////////////////
    
    boolean DEBUG = false;
    
    /**array of box sizes*/
    int[] boxSizes;
    int[] boxesCounted, histogram;
    /**Holds count of number of pixels per box size.*/
    public int[] PixPerBox;
    /**Holds average number of pixels per box size.*/
    public double [] AveragePix;
    /**Holds standard deviations of number of pixels per box size.*/
    public double [] StdDev;
    /**Integer array of arrays for
     * keeping track of the boxes that had
     * pixels over all locations.*/
    public int[][] HadPixAllLocAllSize;
    double[] problac, StdDevOfProbDistributionAtEpsilon,
    MeanOfProbDistributionAtEpsilon;
    /**Holds probability density data.*/
    public double[] mprob;
    
    /**Current rectangle enclosing image.*/
    public java.awt.Rectangle rec;
    /**Coordinates marking where a grid started.*/
    public int actualx=0, actualy = 0;
    int maxImageWidth = 12000;
    
    /**Size of border to add around images.*/
    public int border = 0;
    
    /**Specifies the transparency of the graphic for subareas in
     * {@link #drawit drawit method}.*/
    public float alpha = 0.5f;
    
    /**Stores maximum of
     * pixelated area's width or height*/
    public double maxAcross;
    
    /**Working variable for number of box sizes that is used.*/
    public int newNumSizes;
    
    /**Integer for limit on number of boxes.*/
    public int limit = 4;
    
    int boxes;
    
    /**Holds values for each colour for each square for doing sub areas.*/
    public double [] colourarray;
    /**Coordinates of each origin for coloured squared for doing sub areas.
     * @see #drawNewImage
     */
    public int [] xarray;
    /**Coordinates of each origin for coloured squared for doing sub areas.
     * @see #drawNewImage
     */
    public int []  yarray;
    int left, top, right, bottom;
    int width,height;  //width and height of window
    int topxc, rightyc, leftyc, bottomxc;
    int imgHeight, imgWidth; //dimensions of pixelated area
    long polyWidth, polyHeight, polyLeft, polyTop;
    double totalcountsforallsizes = 0;
    
    /**Specifies the current grid location.*/
    int locations;
    /** */
    //public int Xlimit=0, Ylimit=0;
    String strOldTitle;
    
    /**Directory where files to be opened are. Is set by
     * {@link #openAndrun}.*/
    public String sdir;
    
    /**Holds values for box count data.*/
    public String sizestring;
    
    /**Holds values for box count data.*/
    public String countstring;
    
    String StdDevstring, averagestring,
    problacstring,MeanOfProbDistributionAtEpsilonString,
    StdDevOfProbDistributionAtEpsilonString;
    
    /**Value of pixels that will be assessed.  Is
     *determined in {@link #ready}.*/
    public int foreground;
    
    double margHeight, margWidth;
    double highestx, highesty;
    double margleft, margright, margtop, margbottom;
    
    /**Is reused for each image processed. Is set by
     * {@link #openAndrun}.
     * @see #getCircularityOfConvexHull
     * @see #dosubs    */
    public ImagePlus img;
    
    /**Is reused for each image processed. Is set by
     * {@link #openAndrun}.
     * @see #getCircularityOfConvexHull
     * @see #dosubs  */
    public ImageProcessor ip;
    
    /**Is reused for each image. Is set by
     * {@link #openAndrun}.*/
    public ij.gui.ImageWindow win;
    
    ij.process.ImageStatistics stats;
    
    ///////////////////////////////////////////////////////////////////
    ///////////////// variables set by getInputs method ///////////////
    ///////////////////////////////////////////////////////////////////
    
    /**Number of times to do an entire series of grids (i.e.,
     * if multiple origins are being used).  If set to 1,
     * only one location is used.  Is the number of locations
     * to start a grid series at,
     * rather than the number of box sizes or boxes in each grid series.
     * Is set by {@link #getInputs}.*/
    public int numlocations = 1;
    
    /**Maximum number of box sizes to use in a series.
     * Is set by {@link #getInputs}. Default is 500.*/
    public int incs = 500;
    
    /**Size of boxes to use if doing subareas. Is set by
     *{@link #getInputs}.*/
    public int subboxsize = 20;
    
    /**True shows graphs of regression lines. Is set by {@link #getInputs}.*/
    public boolean viewplots=false;
    
    /**Maximum percent of image area to use as
     *box size.  Based on smallest rectangle
     *enclosing the pixelated part of the image.
     * Is set by {@link #getInputs}.*/
    public double maxpercent=30;
    
    /**True finds
     *fractal dimension over several subareas
     *of the image. False finds it over only
     *the entire area. Is set by {@link #getInputs}.*/
    public boolean dosubs = false;
    
    /**True runs automatic thresholding. Is set by {@link #getInputs}.*/
    public boolean dosomethreshes = false;
    
    /**Number of subareas to analyze if doing a random sample.
     *Is set by {@link #getInputs}.*/
    public int subsamples = 0;
    
    /**"Y" prints summary data. Is set by {@link #getInputs}.*/
    public String printsummary = "y";
    String tab = "\t";
    
    /**If set to 1, runs circularity calculations. Default
     *is 0, to not run them. Is set by {@link #getInputs}.*/
    public int docircul = 0;
    
    /**True fills coloured squares if subareas are being done.
     * Is set by {@link #getInputs}.*/
    public boolean fill=true;
    
    /**True checks a random sample of subareas over the image.
     * Is set by {@link #getInputs}.*/
    public boolean randomsquares=true;
    
    /**True shows a graphic colour coded for the fractal
     *dimension of each subarea. Is set by {@link #getInputs}.*/
    public boolean showcolours = false;
    
    /**True if the user wants to set the number of boxes
     *rather than use a value optimized for each image size.
     * Is set by {@link #getInputs}.*/
    public boolean userNumber=false;
    
    /**True does entire image area. Is set by {@link #getInputs}.*/
    public boolean doglobal=true;
    
    
    ///////////////////////////////////////////////////////////////
    //////////////////////  returned variables  ///////////////////
    ///////////////////////////////////////////////////////////////
    
    /**Perimeter of convex hull.*/
    public double perimeter;
    /**Area of convex hull.*/
    public double AREA = 0;
    /**Circularity of convex hull.*/
    public double circularity;
    /**Variable for roundness of convex hull.*/
    public double roundness;
    /**Centre coordinate of pixelated area.*/
    public double centrex, centrey;
    /**Vertical axis of oval enclosing pixelated area.*/
    public double vDiameter;
    /**Horizontal axis of oval enclosing pixelated area.*/
    double hDiameter ;
    /////////////////END VARIABLE DECLARATIONS////////////////////////
    //////////////////////////////////////////////////////////////////
    
    //////////////////////////////////////////////////////////////////
    /////////////////////          METHODS         ///////////////////
    //////////////////////////////////////////////////////////////////
    /**Runs FracLac.
     * Calls {@link #getInputs getInputs} method.
     * If this returns true, initializes
     * column headings for IJ results window
     * and calls {@link #openAndrun} method, which
     * depends on the value in {@link #randomsquares}. Otherwise terminates.
     * Calls IJ.register on this class.
     *
     * @param arg string not yet used
     */
    public void run(String arg)
    {
        if (getInputs())
        {
            IJ.setColumnHeadings(makeheadings());
            openAndrun(randomsquares);
        }
        IJ.register(FracLac_2003AugustC.class);
    }
    /************************************************************/
    /** Gets circularity of convex hull for passed ImageProcessor.
     * Is called by {@link #writeFinal  writeFinal} method.
     * Calls {@link #GetDimensions GetDimensions} method to find image coordinates,
     * then calls {@link #getPolygon getPolygon} method to make an IJ polygon roi and
     * sets this as the current roi.  Uses IJ analyzer and ResultsTable
     * to change variables for
     * {@link #roundness}, {@link #circularity}, {@link #AREA area},
     * and {@link #perimeter}.
     * <p>circularity = (perimeter*perimeter)/area
     * </p><p>If perimeter is 0 roundness = 0;
     * otherwise roundness = 4.0*pi*(area/(perimeter*perimeter));
     *</p>
     * @param ip ImageProcessor of binary image to make convex hull for*/
    public void getCircularityOfConvexHull(ImageProcessor ip)
    {
        GetDimensions(ip);
        ij.gui.PolygonRoi poly = getPolygon(ip);
        img.setRoi(poly);
        ij.plugin.filter.Analyzer ana = new ij.plugin.filter.Analyzer();
        int ms = 0;
        ms |= ana.AREA+ana.PERIMETER;
        ana.setMeasurements(ms);
        stats = img.getStatistics(ms);
        ij.gui.Roi roi = img.getRoi();
        ana.saveResults(stats, roi);
        ij.measure.ResultsTable rt =ana.getResultsTable();
        int cr = rt.getCounter();
        AREA = rt.getValue(rt.AREA, cr-1);
        perimeter = rt.getValue(rt.PERIMETER, cr-1);
        circularity = (perimeter*perimeter)/AREA;
        if (perimeter==0) roundness = 0;
        else roundness = 4.0*Math.PI*(AREA/(perimeter*perimeter));
        
    }
    
    /************************************************************/
    /**Opens and processes selected files.
     *
     *Reuses global {@link #img} variable for each opened image.  Stores
     *file directory as {@link #sdir}.  Calls {@link #ready ready method}
     * and sets
     *{@link #win} to current image. Makes a new {@link #colourarray},
     *{@link #xarray}, and {@link #yarray} for the image's data,
     * and when done closes {@link #win} if no colour-coding is being shown.
     *Calls {@link #runFL runFL} on {@link #ip} if {@link #doglobal} is true,
     * and {@link #runSubs runSubs} if {@link #dosubs} is true.
     * @param DoRandomSquares boolean true to do random sample
     * instead of entire grid of subareas*/
    public void openAndrun(boolean DoRandomSquares)
    {
        ////////////////////variables///////////////////////////
        double subswide = 0, subshigh = 0;/*for height and width of subareas*/
        javax.swing.JFileChooser fc = null;
        double returnVal;/*for file chooser*/
        java.io.File[] files;/*array of files to be opened*/
        ij.io.Opener opener;/*opener for opening images*/
        int TotalSubAreas;/*for how many subareas to do*/
        ////////////////////////////////////////////////////////
        ////////// select files with a file chooser   //////////
        ////////////////////////////////////////////////////////
        try
        {fc = new javax.swing.JFileChooser();}
        catch (Throwable e)
        {
            IJ.error("This plugin requires Java 2 or Swing.");
            return;
        }/*if the file chooser won't work, stop*/
        if (showcolours==false) fc.setMultiSelectionEnabled(true);
        /*allow more than one file to be selected at a time
         only if the option to show colour is not selected*/
        returnVal = fc.showOpenDialog(IJ.getInstance());
        /*store an input from the user about wht to do*/
        if (returnVal!=javax.swing.JFileChooser.APPROVE_OPTION)
            return;
        /*if the user wants to open, then do it*/
        files = fc.getSelectedFiles();
        /*store the list of files to be opened*/
        if (files.length==0)
        { // getSelectedFiles does not work on some JVMs
            files = new java.io.File[1];
            files[0] = fc.getSelectedFile();
        }
        sdir = fc.getCurrentDirectory().toString();
        /*save the name of the directory for later*/
        opener = new ij.io.Opener();
        /*make a file opener to use with the analysis routines*/
        ///////////////////////////////////////////////////////
        //////////   process the files selected      //////////
        ///////////////////////////////////////////////////////
        for (int i=0; i<files.length; i++)
            /*goes through the list of files one by one*/
        {	img = opener.openImage(sdir, files[i].getName());
                /*opens the file*/
                if (img!=null)
                {/*if it exists, records its processor*/
                    ip = img.getProcessor();
                    img.show();/*show it*/
                    win = ij.WindowManager.getCurrentWindow();
                    /*stores this window's name*/
                    ip = ready(ip);
                    /*runs routine to prepare image*/
                    ///////////////////////////////////////////////////////
                    //////////   Set up sub areas if selected    //////////
                    ///////////////////////////////////////////////////////
                    if (dosubs)
                    {
                        if (subboxsize<=1)subboxsize=2;
                        /*sets local variables for size of the subarea*/
                        // if ((subboxsize*2)>maxAcross)
                        //subboxsize = (int)(maxAcross/2);
                        /*calculates number of subarea boxes there are
                         in the rectangular space of the image*/
                        subswide
                        = Math.ceil((double)rec.width/(double)subboxsize);
                        subshigh
                        = Math.ceil((double)rec.height/(double)subboxsize);
                        TotalSubAreas = (int)(subswide*subshigh);
                        /*calculates number of subareas there will be*/
                        if (DoRandomSquares)TotalSubAreas = subsamples;
                        else subsamples=TotalSubAreas;
                        /*If only a sample is selected, use
                         *the number of samples input by user*/
                        colourarray= new double[subsamples];
                        xarray= new int[subsamples];
                        yarray= new int[subsamples];
                    }//end set sub areas
                    
                    ///////////////////////////////////////////////////////
                    //////////          Do entire area           //////////
                    ///////////////////////////////////////////////////////
                    if (doglobal==true)runFL(0,true, false, ip, rec.x, rec.y);
                    
                    ///////////////////////////////////////////////////////
                    //////////      Do subareas if selected      //////////
                    ///////////////////////////////////////////////////////
                    if (dosubs==true)runSubs(DoRandomSquares);
                    
                    ///////////////////////////////////////////////////////
                    // Close the image if no coloured graphic is made  ////
                    ///////////////////////////////////////////////////////
                    if (showcolours==false)win.close();
                }//end if there is an image
        }//end for to go through list of images
    }//End openAndrun
    /************************************************************/
    /**Finds local fractal dimensions over portions of an image.
     * Calls {@link #stdboxsizes stdboxsizes} to make a series of sizes starting
     *at {@link #subboxsize}*2.  If doing a random sample, resets
     *the roi to a random location once for each {@link #subsamples},
     * otherwise covers the whole image using
     *{@link #rec} for the margins. In both cases, calls {@link #runFL} at each
     *starting origin over the image. Calls {@link #drawNewImage drawnewimage}
     *if {@link #showcolours} is true.
     *  @param DoRandomSquares boolean True does a random sample; false does the
     *entire image
     */
    public void runSubs(boolean DoRandomSquares)
    {/*if the user wants to do sub origins*/
        newNumSizes = stdboxsizes(subboxsize*2);/*make a new set of boxes*/
        java.util.Random rn = new java.util.Random();
        int z = 0, subx=0, suby=0;
        if (DoRandomSquares == true)
        {
            for (int j = 1; j<=subsamples; j++)
                /*do each requested sub analysis at a random start*/
            {
                subx = rec.x-subboxsize+ rn.nextInt(rec.width+subboxsize);
                suby = rec.y-subboxsize + rn.nextInt(rec.height+subboxsize);
                
                do
                {
                    
                    
                    if (subx>(rec.x+rec.width))
                    {subx=rec.x-subboxsize; suby=suby+1;}
                    if (suby>(rec.y+rec.height))suby=rec.y-subboxsize;
                    ip.setRoi(subx, suby, subboxsize, subboxsize);
                    histogram = ip.getHistogram();
                    z = histogram[foreground];
                    if (z==0)subx=subx+1;
                } while (z==0) ;
                runFL(j, false, true, ip, subx, suby);
            }
        }
        else if (DoRandomSquares==false)
        {
            subx=rec.x;//IJ.showMessage("x" + subx);
            suby=rec.y;
            int number = 0;
            for (subx=rec.x; subx<rec.x+rec.width; subx=subx+subboxsize)
                /*do systematic sub analysis */
            {
                
                for (suby=rec.y; suby<rec.y+rec.height; suby=suby+subboxsize)
                {
                    number++;
                    runFL(number, false, true, ip, subx, suby);
                    
                }
            }
        }
        if (showcolours)
        {
            img = ij.WindowManager.getCurrentImage();
            ImageProcessor newip
            = drawNewImage(img, subsamples, subboxsize, xarray, yarray, colourarray);
            img.setProcessor("", newip);
            
            img.setProcessor("", newip);
        }
        
    }
    /************************************************************/
    
    /**Makes a new
     * BufferedImage using width and height of passed image,
     * and draws that passed image and colours on it by calling
     * {@link #drawit drawit method}.
     *
     * @param imp the ImagePlus to draw on
     * @param numShapes the number of squares to draw
     * @param subboxsize the size of each square
     * @param x array of integers for x coordinate of each square
     * @param y array of integers for y coordinate of each square
     * @param colours array of doubles specifying colours
     * @return ImageProcessor with coloured squared drawn on it  */
    public ImageProcessor drawNewImage(
    ImagePlus imp, int numShapes, int subboxsize, int []x, int []y, double [] colours)
    
    {/**Draws one shape for each area that was analyzed,
     * and colours it acccording to the dF at that area*/
        int width = imp.getWidth();
        int  height = imp.getHeight();
        java.awt.image.BufferedImage bi
        = new java.awt.image.BufferedImage(width, height, java.awt.image.BufferedImage.TYPE_INT_ARGB_PRE);
        /**Create a buffered image with translucency*/
        java.awt.Graphics2D g = (java.awt.Graphics2D)bi.getGraphics();
        g.drawImage(imp.getImage(), 0, 0, null);
        /*copy the image*/
        
        drawit(width, height, subsamples, g, subboxsize, x, y, colours);
        /**put the shapes on */
        ImagePlus newimg = new ImagePlus("", bi);
        //newimg.show();/*make a new image*/
        ImageProcessor newip = newimg.getProcessor();
        return newip;
    }
    /************************************************************/
    
    /** Draws differently coloured squares as specified over an image.
     * Uses {@link #alpha} to determine transparency of coloured squares.  Adds
     * a legend bar telling what each colour represents.
     * @param s integer for size of drawing squares
     * @param numShapes the number of squares to draw
     * @param x array of integers for x coordinate of each square
     * @param y array of integers for y coordinate of each square
     * @param colours array of doubles specifying colours
     * @param w integer for width of image
     * @param h integer for height of image
     * @param g2 Graphics2D on which to draw */
    public void drawit(
    int w,
    int h,
    int numShapes,
    java.awt.Graphics2D g2,
    int s,
    int[] x,
    int[] y,
    double[] colours
    )
    /**For drawNewImage, puts the shapes over the original image*/
    {
        
        double firstcolour = 0.0f, lastcolour = 2.0f, sep = 1;
        FracLac_2003AugustC.ColourRule r
        = new FracLac_2003AugustC.ColourRule();
        boolean specificgradient=false;
        
        if (specificgradient)
        {
            firstcolour = colours[0];
            lastcolour = colours[0];
            for (int a = 0; a < colours.length; a++)
            {
                lastcolour = Math.max(colours[a], lastcolour);
                firstcolour = Math.min(colours[a], firstcolour);
            }
            
        }
        
        sep = lastcolour-firstcolour;
        sep = sep/256f;//.0078125
        Color c ;
        double divider = h/20;
        
        for (int i = 0; i < colours.length; i++)
        {
            int blue = (int)((colours[i]-firstcolour)/sep);
            if (blue>255)blue = 255;
            if (blue < 0)blue = 0;
            g2.setColor(r.makeColourRule(blue));
            
            g2.setComposite(
            java.awt.AlphaComposite.getInstance
            (java.awt.AlphaComposite.SRC_OVER, alpha));
            java.awt.geom.Rectangle2D rect =
            new java.awt.geom.Rectangle2D.Double();
            rect.setRect((float)x[i],(float)y[i], s,s);
            if (fill) g2.fill(rect);
            else g2.draw(rect);
        }
        double m=0f, n=0f;
        int bar = 20;boolean skip=true;
        int wordx =bar, wy = 0;int here = 0;
        for (double k = 0.00f; k<2.00f; k=k+.05)
        {
            int b = (int)((k-firstcolour)/sep);
            g2.setColor(r.makeColourRule(b));
            g2.fillRect(0, here, bar, h/40);
            if (skip)
            {g2.drawString(fnum(k,2), wordx, here);skip=false;}else skip=true;
            here = (int)(here + h/40);
        }
        
    }
    /*******************************************************************************/
    /** Class specifies colours to use for colour coding local fractal
     *  dimensions calculated over many subareas. Is used by {@link #drawit}
     */
    public class ColourRule
    {
        //////////////variables////////////////////
        /**For colours of each subarea.*/
        int r,g,b;
        
        /**Constructor accepts no parameters.*/
        public ColourRule()
        {}
        /**Sets values for colours in local dimensions graphic
         * depending on passed value.
         * @param K integer on which colour decision is made
         * @return Color based on K*/
        public Color makeColourRule(int K)
        {
            b=K;
            if (K>255)K=255;if (K<0)K=0;
            boolean tripped = false;
            if (K>254)
            {r=255; g=0; }//b = K-200;}
            else if (K>243)
            {r=245; g=15; }//b = K-200;}
            else if (K>230)
            {r=235; g=30; }//b = K-100;}
            else if (K>217)
            {r=225; g=45;}
            else if (K>211)
            {r=210; g=60;}
            else if (K>204)
            {r=200; g=75;}
            else if (K>198)
            {r=180; g=90;}
            else if (K>192)
            {r=150; g=105;}
            else if (K>185)
            {r=135; g=120;}
            else if (K>179)
            {r=120; g=135;}
            else if (K>172)
            {r=105; g=150;}
            else if (K>166)
            {r=90; g=165;}
            else if (K>160)
            {r=75; g=180;}
            else if (K>153)
            {r=60; g=195;}
            else if (K>140)
            {r=45; g=210; }//b=K+40;}
            else if (K>134)
            {r=30; g=225; }//b=K+40;}
            else if (K>128)
            {r=20; g=240;}
            else if (K>115)
            {r=20; g=250;}//90;}
            else if (K>102)
            {r=10; g=251;}//20;}
            else
            {r=0;g=255;}//g=20;}
            if (tripped)K=K-140;
            
            return new Color(r,g,b);
        }
        
    }
    /************************************************************/
    
    /**Does nothing. Returns nothing.*/
    void done()
    {return;}
    /************************************************************/
    /**Thresholds, finds margins, and sets foreground value on
     *image to be analyzed.
     *
     * Changes string for title to {@link #img}'s current
     *title and reinitializes {@link #sizestring} and
     *{@link #countstring} before getting new stats on
     *{@link #img}. If the image is not binary and {@link #dosomethreshes}
     * is true,
     *calls IJ's thresholding function.  Gets stats again then
     *sets {@link #foreground} to 255 or 0, whichever is lesser
     *in the image. </p>
     *
     * <p>Kills any lefover roi then sets
     * {@link #ip} to img's current processor. Calls
     * {@link #FindMargins} and resets {@link #newNumSizes} to
     * value returned by {@link #stdboxsizes} using value in
     * {@link #maxAcross}.  Resets {@link #border} to the
     * greater of 50 or half {@link #numlocations}.
     *<p>Makes a new ImageProcessor by calling {@link #addborder}
     *on {@link #ip}, then calls {@link #FindMargins} to reset
     *values.</p>
     * @return thisip the final ImageProcessor
     *@param ip ImageProcessor on which to act*/
    public ImageProcessor ready(ImageProcessor ip)
    {
        
        strOldTitle = img.getWindow().getTitle();/*original image title*/
        sizestring = "";
        countstring = "";
        stats = img.getStatistics();/*original image stats*/
        if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)
        {
            if (dosomethreshes==true)IJ.run("Threshold");
            // IJ.error("8-bit binary image (0 and 255) required.");
            //done();
        }
        
        stats = img.getStatistics();
        if (stats.histogram[0]>stats.histogram[255])foreground = 255;
        else foreground = 0;
        img.killRoi();/*no roi on original image*/
        ip=img.getProcessor();
        //ImageProcessor thisip = addborder(ip);
        if (!FindMargins(ip)) done(); else img.killRoi();
        if (DEBUG)IJ.showMessage("Found margins once");
        newNumSizes = stdboxsizes(maxAcross);
        border = Math.max(50, (int)(numlocations)*2);//Math.max(limit, (int)(numlocations)*2);
        //Xlimit=border; Ylimit=border;
        ImageProcessor thisip = addborder(ip);
        if (!FindMargins(thisip)) done(); else img.killRoi();
        if (DEBUG)IJ.showMessage("Found margins twice");
        
        return thisip;
    }
    /************************************************************/
    /**Makes a set of random x and y values within bounds x and y.
     *Size of array is value
     *currently in {@link #numlocations} X 2.
     *@param x integer for maximum x
     *@param y integer for maximum y
     *@return xy integer array of random coordinates
     */
    public int[][] makerandset(int x, int y)
    { int [][]xy = new int [numlocations][2];
      java.util.Random rnd = new java.util.Random();
      for (int i = 0; i < numlocations; i++)
      {
          xy[i][0]=rnd.nextInt(x-1);
          xy[i][1]=rnd.nextInt(y-1);
      }
      return xy;
    }
    
    
    /////////////////////////////////////////////////////////////////////
    /**C
     * @param thisiteration integer for the location currently being done
     * @param circ boolean true calculates convex hull parameters
     * @param issub boolean true does local fractal dimensions
     * @param ipk ImageProcessor ipk is analyzed
     * @param x integer for x coordinate to start at
     * @param y integer for y coordinate to start at
     */
    public void runFL(
    int thisiteration,
    boolean circ,
    boolean issub,
    ImageProcessor ipk,
    int x,
    int y)
    {
        
        double dddf=0;
        String suborglobal = "global";
        if (issub==true) suborglobal = "local";
        String origins = " ("+fnum(x)+ ", "+fnum(y)+ ")";
        /*set this string to actual x and y given*/
        HadPixAllLocAllSize = new int[numlocations + 1][boxSizes.length];
        if (DEBUG)IJ.showMessage("Made all counts array");
        /**Initialize HadPixAllLocAllSize global array */
        for (int j = 0; j < HadPixAllLocAllSize.length; j++)
        {
            for (int k = 0; k < HadPixAllLocAllSize[0].length; k ++)
                HadPixAllLocAllSize[j][k]=0;
        }
        FracLac_2003AugustC.StoreAvgs avs = new FracLac_2003AugustC.StoreAvgs();
        int [][]XY = makerandset(x, y);
        for (locations = 1; locations <= numlocations; locations++)
        {/**for each multiple origin call doBoxCounts function and calculate Df */
            sizestring = "";
            countstring = "";
            StdDevstring = "";
            averagestring = "";
            problacstring = "";
            MeanOfProbDistributionAtEpsilonString="";
            StdDevOfProbDistributionAtEpsilonString="";
            String msg = "Doing locations at " + Integer.toString(locations);
            if (DEBUG)IJ.showMessage(msg);
            
            double sumSC = 0;
            double sumS = 0;
            double sumC=0;
            double sumSsq = 0, sumCsq=0;
            double coVar = 0;
            double avproblac = 0;
            // IJ.showMessage("x "+ x);
            //int [] RS = MakeRandomOrigins(x, y, avs);
            //x=RS[0];
            //y=RS[1];
            if((issub)||(numlocations<2));
            else
            {
                x=XY[locations-1][0];
                y = XY[locations-1][1];
            }
            avs.boxoriginXs[locations-1]=x;
            avs.boxoriginYs[locations-1]=y;
            if (DEBUG)IJ.showMessage("going to do box");
            doBoxCounts(issub, ipk, x, y);
            if (DEBUG)IJ.showMessage("Did BoxCounts");
            /*this depends on the value in locations when it calls count*/
            /**sets values in arrays for pixel data*/
            for (int i = 0; i < newNumSizes; i++)
            {
                /*now go through once for each size to tally the data just stored*/
                sizestring = sizestring + boxSizes[i] + "\t";
                /*make a string of box sizes*/
                countstring = countstring + HadPixAllLocAllSize[locations][i] + "\t";
                /*make a string of numbers of boxes having pixels per box size*/
                averagestring = averagestring + fnum(AveragePix[i]) + "\t";
                StdDevstring = StdDevstring + fnum(StdDev[i]) + "\t";
                problacstring = problacstring + fnum(problac[i]) + "\t";
                MeanOfProbDistributionAtEpsilonString
                =MeanOfProbDistributionAtEpsilonString
                +MeanOfProbDistributionAtEpsilon[i] + "\t";
                StdDevOfProbDistributionAtEpsilonString=StdDevOfProbDistributionAtEpsilonString+
                StdDevOfProbDistributionAtEpsilon[i] + "\t";
                avproblac = avproblac + problac[i];
                coVar = coVar + (StdDev[i]*StdDev[i])/(AveragePix[i]*AveragePix[i]);/*modified coefficient of variation running sum*/
            }
            
            avproblac = avproblac / newNumSizes; double stddevp = 0;/*is sum over num so is mean*/
            for (int j = 0; j < newNumSizes; j++)
            { stddevp = stddevp + ((problac[j] - avproblac)*(problac[j]-avproblac));}
            //add up all the differences squared for prbo density lac
            stddevp = stddevp/((double)newNumSizes-1.0);//variance for prob density lac
            FracStats thisl=new FracStats( HadPixAllLocAllSize[locations], boxSizes, newNumSizes);
            win = ij.WindowManager.getCurrentWindow();
            /*fractal dimension is slope of linear regression line times -1*/
            coVar = coVar/(double)newNumSizes; /*average coefficient of variance*/
            /*store for averages*/
            avs.dfs[locations-1] = thisl.df;
            avs.covars[locations-1] = coVar;
            avs.lacs[locations-1] = thisl.yint;
            avs.cvprob[locations-1]=Math.sqrt((stddevp))/avproblac;
            
            //coefficient of variation for prob den lac string
            if (printsummary == "n")
            {/*print data for each location if user selects this option*/
                String ssddd = suborglobal + " " + avs.boxoriginXs[locations-1]+ ", "
                + avs.boxoriginYs[locations-1]+" " +
                thisiteration+ "." +locations+" ("+strOldTitle+")"+"\t";
                IJ.write(ssddd +fnum(thisl.df)+tab+fnum(thisl.rsq)+ tab+
                fnum(thisl.cdf)+tab+fnum(thisl.crsq)+tab+fnum(coVar)
                +tab+fnum(thisl.yint)+"\t"
                +fnum(avs.cvprob[locations-1]));
                
            }
            if (DEBUG)IJ.showMessage("About to savecolour");
            if (issub)savecolour(thisl.df, thisiteration);
            
        }/*end for locations*/
        finishdata(thisiteration, origins, ipk, avs, circ, issub, suborglobal);
    }
    /******************************************************************/
    /**Calls functions to findMinCover and
     *findAvgCover fractal dimensions.  Makes FracStats instances then calls
     * {@link #writeFinal} method to finish writing data.
     * Calls internal logplotter function if appropriate
     *to view regression line plots.
     * @param thisiteration integer for keeping track of scans
     * @param origins String of x and y where grid started
     * @param ipk ImageProcessor on which to work
     * @param avs StoreAvgs instance
     * @param circ boolean true means do convex hull stats
     * @param issub boolean true means this is a subarea
     * @param suborglobal string for denoting subarea of global */
    public void finishdata(
    int thisiteration,
    String origins,
    ImageProcessor ipk,
    StoreAvgs avs,
    boolean circ,
    boolean issub,
    String suborglobal)
    {
        int [] mincover = findMinCover(HadPixAllLocAllSize, numlocations, newNumSizes);
        /*returns an array of most efficient covering using all locations*/
        float[] avgcover = findAvgCover(HadPixAllLocAllSize, numlocations, newNumSizes);
        /*make an array averaged from all locations*/
        FracStats mins = new FracStats(mincover, boxSizes, newNumSizes);
        FracStats avgs = new FracStats(avgcover, boxSizes, newNumSizes);
        /*make data compressor for this set of locations*/
        
        double AVDF = writeFinal(avgs, mins, ipk, avs,
        circ, issub, suborglobal, thisiteration, origins);
        FracLac_2003AugustC.LogPlotter g =
        new FracLac_2003AugustC.LogPlotter(strOldTitle);
        if((dosubs==false)&&(viewplots==true))
        {int J=2;
         g.set1(avgs, "Avg Df over all locations ", "Avg with horizontal slope removed ");
         
         if (numlocations>1)
         {   J=4;
             g.set2(mins,"Minimum cover Df ", "Min with horizontal slope removed ");
         }
         g.startPlot(J);
        }
    }
    /**********************************************************/
    /**Class for calculating fractal dimension and other stats.*/
    class OldFracStats
    {
        //////////////////variables//////////////////////////////////
        float[] oldcount;
        int[]oldsize;
        double [] CDF;
        int correctedNumBoxes = 0, oldnum;
        int CSize[];
        float CCount[];
        double df, yint, cdf, cyint, rsq, crsq;
        ///////////////////////constructors///////////////////////////
        /**Constructor for integer arrays. Calls
         *{@link #FindDfStats FindDfStats method}
         * and {@link FindCompressed FindCompressed method} on passed
         * arrays. Read {@link #df}, {@link #rsq}, {@link #yint},
         * {@link #cdf}, {@link #crsq}, and {@link #cyint} for
         * values.
         * @param C integer array, usually of counts
         * @param S integer array, usually of sizes
         * @param originalNumSizes integer for number of values to use  */
        public OldFracStats( int[]C, int []S, int originalNumSizes)
        {
            oldcount = new float [originalNumSizes];
            for (int w = 0; w<originalNumSizes; w++)
            {oldcount[w]=(float)C[w];}
            
            this.oldsize=S;
            this.oldnum=originalNumSizes;
            double [] reg = FindDfStats(oldcount, oldsize, originalNumSizes);
            df= reg[0];
            rsq=reg[1];
            yint=reg[2];
            double [] comp = FindCompressed(oldcount, oldsize, originalNumSizes);
            cdf= comp[0];
            crsq=comp[1];
            cyint=comp[2];
        }
        /**Constructor for a float and an integer array. Calls
         *{@link #FindDfStats FindDfStats method}
         * and {@link FindCompressed FindCompressed method} on passed
         * arrays. Read {@link #df}, {@link #rsq}, {@link #yint},
         * {@link #cdf}, {@link #crsq}, and {@link #cyint} for
         * values.
         * @param C float array of counts
         * @param S integer array of sizes
         * @param originalNumSizes integer for number of values to use */
        public OldFracStats( float[]C, int []S, int originalNumSizes)
        {
            this.oldsize=S;
            this.oldcount=C;
            this.oldnum=originalNumSizes;
            double [] reg = FindDfStats(oldcount, oldsize, originalNumSizes);
            df= reg[0];rsq=reg[1];yint=reg[2];
            double [] comp = FindCompressed(oldcount, oldsize, originalNumSizes);
            cdf= comp[0];crsq=comp[1];cyint=comp[2];
            
        }
        ////////////////////////methods/////////////////////////////////////
        /**Converts counts to floats array then
         * calls methods to make and use compressed array.
         *
         * @param intCount integer array of counts
         * @param intSizes integer array of sizes
         * @param oldNmSzs int for number of sizes to use in passed arrays
         * @return  double array from {@link #FindCompressed function} given
         * an array of floats*/
        public double[] FindCompressed(int [] intCount, int[] intSizes, int oldNmSzs)
        {
            float []floatcount = new float [oldNmSzs];
            for (int w = 0; w<oldNmSzs; w++)
            {floatcount[w]=(float)intCount[w];}
            return FindCompressed(floatcount, intSizes, oldNmSzs);
        }
        /********************************************************************/
        /**Removes horizontal slopes from array then finds fractal dimensions.
         *Retains lowest value over horizontal period.
         * @param testarray float array of counts
         * @param Sizes integer array of sizes
         * @param oldNumSizes integer for number of sizes from
         * passed array to use
         * @return double array returned by {@link FindDfStata method}*/
        public double[] FindCompressed(float [] testarray, int[] Sizes, int oldNumSizes)
        {/*shrinks array then calls function to find stats*/
            /*calculates corrected fractal dimension without horizontal slopes*/
            /*uses lowest box size*/
            correctedNumBoxes = 0;
            double CsumSC=0, CsumS=0, CsumC=0, CsumSsq = 0, CsumCsq;
            CSize=new int [oldNumSizes];
            CCount = new float [oldNumSizes];
            for (int q=0; q<oldNumSizes; q++)
                //for (int q=NumSizes-1;q>=0; q--)
            {
                boolean proceed = false;
                if (q>0)
                {
                    if (testarray[q]<testarray[q-1])proceed=true;
                }
                if (proceed==true)
                {
                    CSize[correctedNumBoxes]=Sizes[q];
                    CCount[correctedNumBoxes]=testarray[q];
                    correctedNumBoxes++;
                }
            }
            return FindDfStats(CCount, CSize, correctedNumBoxes);
            
        }
        
        /**Returns a 3 member double array of statistics.  The
         *fractal dimension is the slope of the regression line
         * for the log-log plot of the passed arrays.
         * @param Counts float array of counts
         * @param Sizes integer array of sizes
         * @param NumSizes integer for number of values to use from
         *passed arrays
         * @return double array of fractal dimension, r squared, and
         * y intercept for array*/
        
        public double[] FindDfStats(float[]Counts, int[] Sizes, int NumSizes)
        {  /*takes an array for box sizes and an array for Counts and
         *the number of box sizes to check and calculates stats using those arrays*/
            
            double DsumSC=0, DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
            for (int q=0; q<NumSizes; q++)
            {/*go through array and sum values for regression line*/
                DsumSC = DsumSC + (Math.log((double)Sizes[q])*Math.log(Counts[q]));
                DsumS = DsumS + Math.log((double)Sizes[q]);
                DsumC = DsumC + Math.log((double)Counts[q]);
                DsumCsq=DsumCsq+(Math.log((double)Counts[q])*Math.log(Counts[q]));
                DsumSsq = DsumSsq + (Math.log((double)Sizes[q])*Math.log(Sizes[q]));
            }
            
            double adf= -1*(((double)NumSizes*DsumSC)-(DsumS*DsumC))/(((double)NumSizes*DsumSsq)-(DsumS*DsumS));
            /**adf is the slope of the regression line*/
            double arSq= (NumSizes*(DsumSC) -(DsumS*DsumC))/Math.sqrt(((NumSizes*DsumSsq)-(DsumS*DsumS))*((NumSizes*DsumCsq)-((DsumC*DsumC))));
            arSq=arSq*arSq;
            double ayint = (DsumC - (-1*adf*DsumS))/NumSizes;/*y intercept for linear regression by least squares*/
            
            double [] DFinfo = new double[3];
            DFinfo[0]=adf;
            DFinfo[1]=arSq;
            DFinfo[2]=ayint;
            return DFinfo;
            
            
        }
        /**Finds stats by calling {@link #FindDfStats method} on
         * counts array converted to float array.
         * @param Counts integer array of counts
         * @param Sizes integer array of box sizes
         * @param NumSizes integer for number of values to use
         * @return FindDfStats double array listing stats*/
        public double []FindDfStats(int[]Counts, int[] Sizes,  int NumSizes)
        {/*converter for int array instead of float array*/
            float[]c=new float[NumSizes];
            for (int w = 0; w<NumSizes; w++)  c[w] = (float)Counts[w];
            return FindDfStats(c,Sizes,NumSizes);
        }
    }/*end inner class*/
    /**************************************************************/
    
    /************************************************************/
    class LogPlotter
    {/*class to plot and compare data arrays*/
        float []x,y,x2,y2,x3, y3, x4, y4;
        float []yr,y2r, y3r, y4r;
        double r, r2, r3, r4;
        double df, df2, df3, df4, yint, yint2, yint3, yint4;
        String filename, dfname, df2name, df3name, df4name;
        double xmax, xmin, ymax, ymin;
        PlotWindow plot;
        LogPlotter(String f)
        {filename=f;}
        
        void set1(FracStats F, int []xints, int []yints, int xnum,
        double DF, double yint, String DFNAME)
        {
            
            x = InttoLogFloat(xints, xnum);
            y= InttoLogFloat(yints, xnum);
            df=DF;
            dfname=DFNAME;
            yint=yint;
            yr=new float[xnum];
            for (int k=0; k<xnum; k++)yr[k]=regrY(df, yint, x[k]);
            
        }
        
        void set1(FracStats F, String DFNAME, String DF2NAME)
        {
            x= InttoLogFloat(F.oldsize, F.oldnum);
            y=  LogFloat(F.oldcount, F.oldnum);
            df=F.df;
            r=F.rsq;
            dfname=DFNAME;
            yint=F.yint;
            yr=new float[F.oldnum];
            for (int k=0; k<F.oldnum; k++)yr[k]=regrY(df, yint, x[k]);
            
            
            x2 =InttoLogFloat(F.CSize, F.correctedNumBoxes);
            y2=  LogFloat(F.CCount, F.correctedNumBoxes);
            df2=F.cdf;
            r2=F.crsq;
            df2name=DF2NAME;
            yint2=F.cyint;
            y2r=new float[F.correctedNumBoxes];
            for (int k=0; k<F.correctedNumBoxes; k++)y2r[k]=regrY(df2, yint2, x2[k]);
            
        }
        void set2( FracStats G, String DF3NAME, String DF4NAME)
        {
            x3= InttoLogFloat(G.oldsize, G.oldnum);
            y3=  LogFloat(G.oldcount, G.oldnum);
            df3=G.df;
            r3=G.rsq;
            df3name=DF3NAME;
            yint3=G.yint;
            y3r=new float[G.oldnum];
            for (int k=0; k<G.oldnum; k++)y3r[k]=regrY(df3, yint3, x3[k]);
            
            
            x4 =InttoLogFloat(G.CSize, G.correctedNumBoxes);
            y4=  LogFloat(G.CCount, G.correctedNumBoxes);
            df4=G.cdf;
            r4=G.crsq;
            df4name=DF4NAME;
            yint4=G.cyint;
            y4r=new float[G.correctedNumBoxes];
            for (int k=0; k<G.correctedNumBoxes; k++)y4r[k]=regrY(df4, yint4, x4[k]);
        }
        /****************************************************/
        float regrY(double m, double b, double x)
        {
            return (float)((-1*m*x) + b);
        }
        
        void drawtheplot()
        {plot.draw();}
        void setMaxMin(int J)
        { xmax = x[0]; xmin = x[0]; ymax = y[0]; ymin = y[0];
          
          
          for (int u = 0; u<x.length; u++)
          {
              xmax = Math.max(x[u], xmax);
              xmin = Math.min(x[u], xmin);
          }
          
          for (int u = 0; u<y.length; u++)
          {
              ymax = Math.max(y[u], ymax);
              ymin = Math.min(y[u], ymin);
          }
          for (int u = 0; u<yr.length; u++)
          {
              ymax = Math.max(yr[u], ymax);
              ymin = Math.min(yr[u], ymin);
          }
          
          if (J>1)
          {for (int u = 0; u<x2.length; u++)
           {
               
               xmax = Math.max(x2[u], xmax);
               xmin = Math.min(x2[u], xmin);
               
           }
           for (int u = 0; u<y2.length; u++)
           {
               ymax = Math.max(y2[u], ymax);
               ymin = Math.min(y2[u], ymin);
           }
           for (int u = 0; u<y2r.length; u++)
           {   ymax = Math.max(y2r[u], ymax);
               ymin = Math.min(y2r[u], ymin);
               
           }
          }
          if (J>2)
          {
              for (int u = 0; u<x3.length; u++)
              {
                  xmax = Math.max(x3[u], xmax);
                  xmin = Math.min(x3[u], xmin);
              }
              
              for (int u = 0; u<y3.length; u++)
              {
                  ymax = Math.max(y3[u], ymax);
                  ymin = Math.min(y3[u], ymin);
              }
              for (int u = 0; u<y3r.length; u++)
              {
                  ymax = Math.max(y3r[u], ymax);
                  ymin = Math.min(y3r[u], ymin);
              }
              
          }
          if (J>3)
          {
              for (int u = 0; u<x4.length; u++)
              {
                  xmax = Math.max(x4[u], xmax);
                  xmin = Math.min(x4[u], xmin);
                  
              }
              for (int u = 0; u<y4.length; u++)
              {
                  ymax = Math.max(y4[u], ymax);
                  ymin = Math.min(y4[u], ymin);
              }
              for (int u = 0; u<y4r.length; u++)
              {
                  ymax = Math.max(y4r[u], ymax);
                  ymin = Math.min(y4r[u], ymin);
              }
          }
          
          
        }
        
        
        /************************************************/
        float[] InttoLogFloat(int[] ints, int NumberOfInts)
        {
            float floats[]=new float[NumberOfInts];
            for (int g=0; g<NumberOfInts;g++)
            {
                floats[g]=(float)Math.log((double)ints[g]);
            }
            
            return floats;
        }
        /***************************************************/
        /************************************************/
        float[] LogFloat(float[] fs, int NumberOfInts)
        {
            float floats[]=new float[NumberOfInts];
            for (int g=0; g<NumberOfInts;g++)
            {
                floats[g]=(float)Math.log((double)fs[g]);
            }
            
            return floats;
        }
        /***************************************************/
        void startPlot(int J)
        {
            setMaxMin(J);
            plot = new PlotWindow("Regression Lines "+ filename,
            "log size","log count",x,yr);
            plot.setLimits(xmin, xmax, ymin, ymax);
            plot.addPoints(x,y,PlotWindow.CIRCLE);
            // add a second curve
            plot.addLabel(0,0,dfname + ": " + fnum(df)+"r sq=" + fnum(r));
            if (J>1)plot2();
            if (J>2)plot3();
            if (J>3)plot4();
            plot.draw();
        }
        
        void plot2()
        {
            
            plot.setColor(Color.red);
            plot.addPoints(x2,y2r,PlotWindow.LINE);
            plot.addLabel(0,.2, df2name + " " + fnum(df2)+"r sq=" + fnum(r2));
            
            
        }
        void plot3()
        {
            // add a 3rd curve
            plot.setColor(Color.green);
            //plot.addPoints(x3,y3,PlotWindow.CIRCLE);
            plot.addPoints(x3,y3r,PlotWindow.LINE);
            plot.addLabel(0,.80,df3name+ " " + fnum(df3)+"r sq=" + fnum(r3));
            //plot.addPoints(x3,y3,PlotWindow.LINE);
        }
        void plot4()
        {// add a 4th curve
            plot.setColor(Color.blue);
            //plot.addPoints(x4,y4,PlotWindow.X);
            plot.addPoints(x4,y4r,PlotWindow.LINE);
            
            plot.addLabel(00,1,df4name+" " + fnum(df4)+"r sq=" + fnum(r4));
            
            plot.setColor(Color.cyan);
        }
        
        
    }
    /**************************************/
    float [] findAvgCover(int [][] TestArray, int numLocs, int numBoxes)
    {/*averages all arrays in a 2d array to make one value for each slot*/
        float  avcover[]= new float[numBoxes];
        //IJ.showMessage(fnum((double)TestArray[0][2]));
        /*initialize the array using the first location*/
        /*for each size, average boxes containing pixels from all locations*/
        /*store this in the array*/
        for (int a = 0; a < numBoxes; a++)
        {avcover[a] = 0.0f;}
        for (int j = 1; j<=numLocs; j++)
        {
            for (int i = 0; i < numBoxes; i++)
            {
                avcover[i] =  avcover[i] + (float)TestArray[j][i];
            }
        }
        
        for (int i = 0; i < numBoxes; i++)
        {
            avcover[i] =  avcover[i]/(float)numLocs;
        }
        
        return avcover;
                /*if the number of boxes having pixels is lower at this location
                 than at the lowest recorded location for this size,
                 then store this number at this size*/
        
    }
    /*****************************************************************/
    /**Returns an array holding the minimum value for each position in
     *an array of arrays.
     * @param TestArray integer array of arrays
     * @param numLocs integer for number of arrays to use
     * @param numBoxes integer for number of values in arrays to use
     * @return mcover integer array holding the lowest value at each position*/
    
    public int [] findMinCover(int [][] TestArray, int numLocs, int numBoxes)
    {
        int  mcover[]= new int[numBoxes];
        
        /*initialize the minimum cover array using the first location*/
        /*for each size, find minimum boxes containing pixels from all locations*/
        /*store this in the mincover array*/
        for (int a = 0; a < numBoxes; a++)
        {mcover[a] = TestArray[1][a];}
        for (int j = 2; j<numLocs; j++)
        {
            for (int i = 0; i < numBoxes; i++)
            {
                if (TestArray[j][i]<mcover[i])
                    mcover[i] =  TestArray[j][i];
            }
        }
        return mcover;
    }
    /**********************************************************************/
    /**Writes the final values to the IJ window.
     * @param avgstat FracStats instance holding all original data
     * @param min Fracstats instance holding all data for minimum cover
     * @param ipk ImageProcessor on which to work
     * @param avs StoreAvgs instance holding average values
     * @param circ boolean true includes circularity and other convex hull data
     * @param issub boolean true includes subarea data
     * @param suborglobal string specifies data type
     * @param thisiteration int for subareas or global
     * @param origins String of x and y coordinates
     * @return double average df */
    public double writeFinal(
    FracStats avgstat,
    FracStats min,
    ImageProcessor ipk,
    StoreAvgs avs,
    boolean circ,
    boolean issub,
    String suborglobal,
    int thisiteration,
    String origins)
    {/*only writes after all locations are done*/
        double avdf = 0;
        double avcoVar = 0;
        double avF = 0;
        double avLac = 0;
        double avcvP = 0, sumF = 0, sumFsq=0;
        double sumdf = 0, sumdfsq=0;
        
        for (int m = 0; m < numlocations; m++)
        {
            avdf = avs.dfs[m] + avdf;
            avcoVar = avs.covars[m] + avcoVar;
            avF = avs.lacs[m] + avF;
            avcvP = avs.cvprob[m] + avcvP;
            sumF=sumF + avs.lacs[m];
            sumFsq=sumFsq + (avs.lacs[m]*avs.lacs[m]);
            sumdf=sumdf + avs.dfs[m];
            sumdfsq=sumdfsq + (avs.dfs[m]*avs.dfs[m]);
        }
        double stdevF = (sumFsq - ((sumF*sumF)/numlocations))/(numlocations-1);
        stdevF = Math.sqrt(stdevF);
        double meanF = sumF/numlocations;
        double coVarF = stdevF/meanF;
        double stdevdf = (sumdfsq - ((sumdf*sumdf)/numlocations))/(numlocations-1);
        stdevdf = Math.sqrt(stdevdf);
        double meandf = sumdf/numlocations;
        double coVardf = stdevdf/meandf;
        
        avcvP = avcvP/(double)numlocations;//average covar for prob den lac for all locations
        avF = avF/(double)numlocations;
        double jjj = 0;
        
        for (int m = 0; m < numlocations; m++)
        {
            jjj = jjj + ((avs.cvprob[m]-avcvP) * (avs.cvprob[m]-avcvP));
            avLac = avLac + (Math.pow(((avs.lacs[m]/avF)-1.0), 2));
        }
        jjj = jjj/((double)locations-1.0);//variance
        double cvarP = Math.sqrt(jjj)/avcvP;//coefficient of variation for coef of variation for prob density lacunarity
        
        /*fractal dimension is slope of linear regression line times -1*/
        //coVar = coVar/(double)newNumSizes; /*average coefficient of variance*/
        //double minyint = (minsumC - (df*sumS))/newNumSizes;/*y intercept for linear regression by least squares*/
        
        avdf = avdf/(double)numlocations;
        avcoVar = avcoVar/(double)numlocations;
        avLac = avLac/(double)numlocations;
        
        if ((docircul == 1)&&(circ==true) && (issub==false)) getCircularityOfConvexHull(ipk);
        
        String thistype = suborglobal + " Summary "+thisiteration + origins + " ";
        IJ.write(
        thistype + strOldTitle + "\t"
        + fnum(avgstat.df) + tab
        + fnum(avgstat.rsq) + tab
        + fnum(avgstat.cdf) + tab
        + fnum(avgstat.crsq) + tab
        + fnum(avcoVar) +  "\t"
        + fnum(avLac) +"\t"
        + fnum(cvarP) + "\t"
        + fnum(this.AREA)+ "\t"
        + fnum(this.perimeter)+ "\t"
        + fnum(this.circularity) + "\t"
        + fnum(this.roundness) + "\t"
        + fnum(margWidth)  + "\t"
        + fnum(margHeight)   + "\t"
        + fnum(vDiameter)  +"\t"
        + fnum(hDiameter)  + "\t"
        + fnum(centrex)+"/" +fnum(centrey)    + "\t"
        + fnum(newNumSizes) + tab
        + fnum(avgstat.correctedNumBoxes) + tab
        + fnum(min.df)+ tab
        + fnum(min.rsq) + tab
        + fnum(min.cdf) + tab
        + fnum(min.crsq) + tab
        + fnum(coVarF)     + "\t"
        + fnum(coVardf)    + "\t"
        + fnum(numlocations) + " maxbox= " +limit );
        
        
        if ((dosubs==false)&&(printsummary=="n"))
        {
            
            String locs = "Count at location: 1" + "\t";
            for (int y =2; y<=numlocations; y++) locs = locs + y+"\t";
            IJ.write("Box Size" + "\t" + locs);
            
            for (int m=0; m<newNumSizes; m++)
            {   String L = "";
                L = L + boxSizes[m] + "\t";
                for (int i = 1; i <= numlocations; i++)
                {/*now go through once for each size*/
                    L = L + HadPixAllLocAllSize[i][m] + "\t";
                    /*make a string of numbers of boxes
                     having pixels per box size*/
                }IJ.write(L);
            }
            
        }
        return avdf;
    }
    /********************************************************************/
    class StoreAvgs
    {
        double[] dfs = new double[numlocations];
        double[] covars = new double[numlocations];
        double[] lacs = new double [numlocations];
        double []cvprob = new double[numlocations];
        int []boxoriginXs = new int[numlocations];
        int []boxoriginYs = new int[numlocations];
    }
    
    /********************************************************************/
    /**Adds a border around an image using ImageProcessor functions.
     * @param ipOld THe ImageProcessor to act on
     * @param wNew double for new width
     * @param hNew double for new height
     * @param xOff double for offset from x coordinate
     * @param yOff double for offset from y coordinate
     * @return ipNew ImageProcessor with border*/
    public ImageProcessor expandImage(ImageProcessor ipOld, double wNew, double hNew,
    double xOff, double yOff)
    {/*begin function to add border*/
        ImageProcessor ipNew = ipOld.createProcessor((int)wNew, (int)hNew);
        ipNew.setColor(ij.gui.Toolbar.getBackgroundColor());
        ipNew.fill();
        ipNew.insert(ipOld, (int)xOff, (int)yOff);
        return ipNew;
    }/*end add border*/
    
    /********************************************************************/
    /**Saves the colour in {@link #colourarray}
     * to use at each square for which a local
     * dimension was calculated.  Uses previously stored values
     * in {@link #actualx} and
     * {@link #actualy} to change {@link #xarray} and {@link #yarray}.
     * @param dddf double for the fractal dimension
     * @param iter integer for the number in the list */
    
    public void savecolour(double dddf, int iter)
    {
        colourarray[iter-1]=dddf;
        xarray[iter-1] = actualx;
        yarray[iter-1] = actualy;
    }
    /******************************************************************/
    /**Calculates the array of
     * box sizes.  {@link #limit} is first set to
     * {@link #maxpercent a percentage} of
     * the passed value.  The minimum is 15 boxes.  If the user wants to
     *choose the number of boxes (if {@link #userNumber} is true),
     *number of boxes is set to the lesser of {@link #limit} or
     *the number the user set {@link #incs in the setup}.  The increment
     *between box sizes is set as {@link #limit} divided by number of boxes.
     * <p>Also reinitializes arrays:
     * AveragePix, StdDev, problac, MeanOfProbDistributionAtEpsilon, and
     * StdDevOfProbDistributionAtEpsilon.
     * @param Across double for distance across image
     * @return  int for number of boxes in the series*/
    public int stdboxsizes(double Across)
    {
        
        limit = (int)(Across*(maxpercent/100));
        if (limit<15) limit=15;
        int numboxes = limit;
        if (userNumber) numboxes = Math.min(limit, incs);
        if (numboxes<15) numboxes = 15;
        boxSizes = new int[numboxes];
        int increment = (int)(limit/numboxes);
        //mprob = new double[stats.pixelCount];
        for (int r = 0; r < 15; r++)boxSizes[r] = r+1;
        int i = 15;
        for (i = 15; ((boxSizes[i-1] < limit) && i < numboxes-1 ); i++)
        {
            /*limit the box size to the maximum diameter*/
            
            boxSizes[i] = boxSizes[i-1] + increment;
        }
        boxSizes[i-1]=limit;
        AveragePix = new double[i];
        StdDev = new double [i];
        problac = new double[i];
        MeanOfProbDistributionAtEpsilon=new double[i];
        StdDevOfProbDistributionAtEpsilon=new double[i];
        return i;
    }
    /*****************************************************************/
    /**Finds margins of pixelated part of binary image and sets
     * meassurements for height, width, and span.
     * Uses value set in {@link #foreground}
     * to search for pixels of that colour.
     * @param ip ImageProcessor to work on
     * @return boolean always returns true when done  */
    public boolean FindMargins(ImageProcessor ip)
    {
        int [] histogram = new int[256];
        double totalwidth = ip.getWidth();
        double totalheight = ip.getHeight();
        hDiameter = 0;
        vDiameter = 0;
        //Find left edge
        margleft = -1;
        do
        {	margleft++;
                if (margleft>=totalwidth)
                {
                    IJ.error("No non-backround pixels found.");
                    return false;
                }
                ip.setRoi((int)margleft, 0, 1, (int)totalheight);
                histogram = ip.getHistogram();
        } while (histogram[foreground]==0);
        
        //Find top edge
        margtop = -1;
        do
        {	margtop++;
                ip.setRoi((int)margleft, (int)margtop,
                (int)(totalwidth-margleft), 1);
                histogram = ip.getHistogram();
        } while (histogram[foreground]==0);
        //Find right edge
        margright =totalwidth+1;
        do
        {	margright--;
                ip.setRoi((int)margright-1, (int)margtop, 1,
                (int)(totalheight-margtop));
                histogram = ip.getHistogram();
        } while (histogram[foreground]==0);
        
        //Find bottom edge
        margbottom =totalheight+1;
        do
        {	margbottom--;
                ip.setRoi((int)margleft, (int)margbottom-1,
                (int)(margright-margleft), 1);
                histogram = ip.getHistogram();
        } while (histogram[foreground]==0);
        margHeight = margbottom - margtop;
        margWidth = margright - margleft;
        rec = new java.awt.Rectangle((int)margleft, (int)margtop,
        (int)margWidth, (int)margHeight);
        maxAcross = Math.min(rec.width, rec.height);
        centrex = margleft + (margWidth/2);
        centrey = margtop + (margHeight/2);
        
        //Find longest vertical radius
        highestx = margleft-1;
        do
        {	highestx++;
                ip.setRoi((int)highestx, (int)margtop, 1,1);
                histogram = ip.getHistogram();
                if (histogram[foreground] > 0)
                {
                    double d = Math.sqrt(((highestx-centrex)*(highestx-centrex))
                    + ( (margbottom - centrey )*(margbottom - centrey ) )   );
                    vDiameter = (double)Math.max(vDiameter, d);
                }
        }while (highestx <=margright);
        
        highestx = margleft -1;
        do
        {
            highestx++;
            ip.setRoi((int)highestx, (int)margbottom, 1,1);
            histogram = ip.getHistogram();
            if (histogram[foreground] >0)
            {
                double d = Math.sqrt(((highestx-centrex)*(highestx-centrex))
                +((margbottom-centrey)*(margbottom-centrey)));
                vDiameter = (double)Math.max(vDiameter, d);}
        } while (highestx<=margright);
        vDiameter  = 2*vDiameter;
        
        //Find longest horizontal radius
        highesty = margtop-1;
        do
        {	highesty++;
                ip.setRoi((int)margleft, (int)highesty, 1,1);
                histogram = ip.getHistogram();
                if (histogram[foreground] > 0)
                {
                    double d = Math.sqrt(((margleft-centrex)*(margleft-centrex))
                    +((highesty-centrey)*(highesty-centrey)));
                    hDiameter=(double)Math.max(hDiameter,d);
                }
        }while (highesty<=margbottom);
        
        highesty=margtop-1;
        do
        {	highesty++;
                ip.setRoi((int)margright, (int)highesty, 1,1);
                histogram=ip.getHistogram();
                if (histogram[foreground] >0)
                {
                    double d=Math.sqrt(((margright-centrex)*(margright-centrex))
                    +((highesty-centrey)*(highesty-centrey)));
                    hDiameter = (double)Math.max(hDiameter,d);
                }
        } while (highesty <= margbottom);
        hDiameter  = 2*hDiameter;
        return true;
    }
    /*******************************************************************/
    /**Adds a border to the image. Uses value currently in {@link #border}.
     *Calls {@link #expandImage expandImage} to do the work.
     * @param ip ImageProcessor
     * @return  ImageProcessor with border around it*/
    public ImageProcessor addborder(ImageProcessor ip)
    { /**Adds a border to the image*/
        double wOld, hOld, wNew, hNew;/*variables for borders*/
        wOld = ip.getWidth();/* get the image's width and height*/
        hOld = ip.getHeight();
        wNew = wOld + border;/* set up a new value for the added border*/
        hNew = hOld + border;/*set up a new value for the added border*/
        double xOff = (wNew - wOld)/2;	// offset for centering
        double yOff = (hNew - hOld)/2;	// offset for centering
        ImageProcessor newIP;
        /**make a copy with a new border; based on ImageJ plugin Resize Canvas*/
        newIP = expandImage(ip, wNew, hNew, xOff, yOff);
        win.close();
        new ImagePlus(strOldTitle + " copy", newIP).show();
        return newIP;
    }
    
    /**Sets up and calls {@link #moveGridalloverCountPixels method}.
     * In doing this, changes global variable for boxes and global arrays
     * for number of pixels and density. Is called by {@link #doBoxCounts method}.
     * Changes x and y for main, not sub, checks, for placing
     * the starting grad at
     * systematic multiple origins around the start. For subs,
     * makes the area to check into
     * a smaller rectangle but for mains just leaves the area
     * to check as the whole image.
     *
     * @param asub boolean true if doing subareas
     * @param boxnumber integer for keeping track
     * @param sze integer for size of box
     * @param ip ImageProcessor to work on
     * @param x integer for starting x coordinate
     * @param y integer for starting y coordinate
     * @return boxes double for number of boxes with pixels in them */
    public double count(
    boolean asub,
    int boxnumber,
    int sze,
    ImageProcessor ip,
    int x,
    int y)
    {
        int rit = ip.getWidth();//(int)margright;
        //for main analysis, set the right edge of the total pixelated area
        int botm = ip.getHeight();//(int)margbottom;//set bottom for main
        if (asub)
        {rit = x+limit; botm=y+limit;}
        //for subareas only, reset the boundaries of the
        //area to check using the values for the subarea size
        if (DEBUG)IJ.showMessage("going to call move grid for box number "
        +Integer.toString(boxnumber));
        // IJ.showMessage("it is "+rit);
        
        boxes = moveGridalloverCountPixels(boxnumber, x, y, sze, botm, rit, ip);
        /*count pixels in area of ip using grid of size, and store
         values in global arrays*/
        
        return boxes;/*return the number of boxes that had pixels in them*/
    }
    /***************************************************************/
    String fnum(double sum)
    {
        
        DecimalFormat myFormatter = new DecimalFormat("####.####");
        String output = myFormatter.format(sum);
        return output;
    }
    /***************************************************************/
    /**Formats numbers.
     * @param sum number to format
     * @param decimals number of decimal places
     * @return String spelling formatted number */
    public String fnum(double sum, int decimals)
    {
        
        DecimalFormat myFormatter = new DecimalFormat("####.####");
        switch (decimals)
        {case 0: myFormatter = new DecimalFormat("####.");
            case 1: myFormatter = new DecimalFormat("####.#");
            case 2: myFormatter = new DecimalFormat("####.##");
            case 3: myFormatter = new DecimalFormat("####.###");
            case 4: myFormatter = new DecimalFormat("####.####");
            case 5: myFormatter = new DecimalFormat("####.#####");
            default: myFormatter = new DecimalFormat("####.####");
        }       String output = myFormatter.format(sum);
        return output;
    }
    /***************************************************************/
    /***************************************************************/
    /**Returns a double array holding a probability density distribution
     * for the passed array, using as many elements as the passed number; since
     *it is usually called after each box size is scanned, it is usually
     * for a particular epsilon (box size) at a particular location.
     *The array holds at each element the percent of boxes that had
     * the number of pixels
     * that corresponds to the index of the element, so to access the
     * probability that a box had 3 pixels, get the value at index 3.
     *Calculates statistics from array named mprob:
     * the mean of the probability distribution is: sum of all(i*mprob[i])
     * where i is the index and value, and mprob[i] is the weight
     * the variance of the probability distribution is:
     * sum of all (((mprob[i]-meanProb)*(mprob[i]-meanProb))*i);
     *the Probability Density Lacunarity at this epsilon is:
     * (sumMsq - (sumM*sumM))/(sumM*sumM);
     * where sumM=sum of all(i*mprob[i])
     * and sumMsq = sum of all (i*i*mprob[i]);
     *These are all stored in arrays and written to the final file.
     * @param epsilon integer for the box size; is only used to store final value
     * @param PixelsPerBox integer array of values to be arranged
     * @param numberOfBoxesWithPixels integer for the number of elements
     *
     * /************************************************************************/
    /***************************************************************/
    /** Returns a double array holding a probability density distribution
     * for the passed array, using as many elements as the passed number; since
     * it is usually called after each box size is scanned, it is usually
     * for a particular epsilon (box size) at a particular location.
     * The array holds at each element the percent of boxes that had
     * the number of pixels
     * that corresponds to the index of the element, so to access the
     * probability that a box had 3 pixels, get the value at index 3.
     * Calculates statistics from array named mprob:
     * the mean of the probability distribution is: sum of all(i*mprob[i])
     * where i is the index and value, and mprob[i] is the weight
     * the variance of the probability distribution is:
     * sum of all (((mprob[i]-meanProb)*(mprob[i]-meanProb))*i);
     * the Probability Density Lacunarity at this epsilon is:
     * (sumMsq - (sumM*sumM))/(sumM*sumM);
     * where sumM=sum of all(i*mprob[i])
     * and sumMsq = sum of all (i*i*mprob[i]);
     * These are all stored in arrays and written to the final file.
     * @param epsilon integer for the box size; is only used to store final value
     * @return mprob double array of probabilities of a number of
     * pixels being found in an area*/
    public double [] probmass(int epsilon)//, int [] PixelsPerBox,  int numberOfBoxesWithPixels)
    {
        //makes an array containing the probability
        //of each value in pixperbox array, for a box size
        //uses values already in PixPerbox;
        //just counts how many there were of each value
        
        int numberOfBoxesWithPixels=boxes;
        double sumM = 0;
        double sumMsq = 0;
        int BiggestNumberOfPixels=PixPerBox[0];
        for (int i = 0; i<numberOfBoxesWithPixels; i++)
        {
            BiggestNumberOfPixels
            =Math.max(BiggestNumberOfPixels, PixPerBox[i]);
        }
        
        mprob = new double[BiggestNumberOfPixels+1];
        for (int i = 0; i<mprob.length; i++)
            mprob[i]=0;
        for ( int a = 0;a < numberOfBoxesWithPixels; a++)
        {
            //count number of each value in PixpPerBox, including duplicates
            //mprob will contain an array of how many
            //there were for each entry in PixPerBox
            //instead of the ntry itself
            
            if (true)
            {
                double counter=0;
                for (int i = 0; i < numberOfBoxesWithPixels; i++)
                {
                    if (PixPerBox[a] == PixPerBox[i])counter++;
                }
                mprob[PixPerBox[a]]=counter;
                
            }
        }
        
        //mprob now holds the probability
        //of a number at the corresponding position
        //as well as zeros where there are no pixels
        for (int i = 0; i < mprob.length; i++)
        {//Now mprob contains the number of times
            //a box had that many pixels divided by
            //the number of boxes there were in all
            //so it is the probability of that number of
            //pixels being found in a box
            mprob[i] = mprob[i]/(double)numberOfBoxesWithPixels;
        }
        
        
        for (int i = 0; i < mprob.length; i++)
        {//calculate moments as per chap. 8.4.10 , p. 238,  Innaccone
            
            
            //This is the summed densities of pixels
            sumM=sumM + (i*mprob[i]);
            //This is the squared density
            sumMsq = sumMsq + (i*i*mprob[i]);
            
        }
        double meanProb=sumM;
        double variance=0;
        
        for (int i=0; i<mprob.length; i++)
        {
            if (mprob[i]>0)
            {variance=variance+(((i-meanProb)*(i-meanProb))*mprob[i]);}
            
        }
        
        //robDensityStdDevPerEpsilon[epsilon]=Math.sqrt(variance);
        
        //ProbDensityMeanPerEpsilon[epsilon]=meanProb;
        MeanOfProbDistributionAtEpsilon[epsilon]=meanProb;
        StdDevOfProbDistributionAtEpsilon[epsilon]=Math.sqrt(variance);
        //ProbDensityCoefVariation[epsilon]=Math.sqrt(variance)/meanProb;
        
        //store the probability density lacunarity for this series
        //ProbabilityDensityLacunarityAtEpsilon
        problac[epsilon]=(sumMsq - (sumM*sumM))/(sumM*sumM);
        
        return mprob;
        
    }
    /************************************************************************/
    
    //void aprobmass(int epsilon) {
    ///*makes an array containing the probability of each value in pixperbox array, for a box size*/
    ///*uses values already in PixPerbox; just counts how many there were of each value*/
    // double sumM = 0;
    // double sumMsq = 0;
    // int BiggestNumberOfPixels=PixPerBox[0];
    //  for (int i = 0; i<boxes; i++)
    // {
    //    BiggestNumberOfPixels
    //     =Math.max(BiggestNumberOfPixels, PixPerBox[i]);
    //  }
    //  mprob = new double[BiggestNumberOfPixels+1];
    //  for (int i = 0; i<mprob.length; i++) mprob[i]=0;
    
    //  for ( int a = 0;a < boxes; a++)
    //  {
    //count number of each value in PixpPerBox, including duplicates
    //mprob will contain an array of how many
    //there were for each entry in PixPerBox
    //instead of the ntry itself
    
    //  if (true)
    // {
    //  double counter=0;
    //  for (int i = 0; i < boxes; i++)
    // {
    //    if (PixPerBox[a] == PixPerBox[i])counter++;
    //  }
    //    mprob[PixPerBox[a]]=counter;
    
    //  }
    //}
    //for (int i = 0; i < mprob.length; i++)
    //{//Now mprob contains the number of times
    //a box had that many pixels divided by
    //the number of boxes there were in all
    //so it is the probability of that number of
    //pixels being found in a box
    //  mprob[i] = mprob[i]/boxes;
    //}
    
    //for (int i = 0; i < mprob.length; i++)
    //{//calculate moments as per chap. 8.4.10 , p. 238,  Innaccone
    
    
    //This is the summed densities of pixels
    // sumM=sumM + (i*mprob[i]);
    //This is the squared density
    //   sumMsq = sumMsq + (i*i*mprob[i]);
    
    //}
    // double MeanOfProbabilityDistribution=sumM;
    // double variance=0;
    
    //for (int i=0; i<mprob.length; i++)
    // {
    //if (mprob[i]>0)
    // {variance=variance+(((i-MeanOfProbabilityDistribution)
    //    *(i-MeanOfProbabilityDistribution))*mprob[i]);}
    
    // }
    
    // StdDevOfProbDistributionAtEpsilon[epsilon]=Math.sqrt(variance);
    //   MeanOfProbDistributionAtEpsilon[epsilon]=MeanOfProbabilityDistribution;
    //store the probability density lacunarity for this series
    //     problac[epsilon]
    //  = (sumMsq - (sumM*sumM))/(sumM*sumM);
    
    //return mprob;
    // problac[epsilon] = (sumMsq - (sumM*sumM))/(sumM*sumM);
    
    //  }
    /************************************************************************/
    
    //double CVgridDensity (int n)
    // {//calculates the variation in pixelation density at
    //a certain epsilon; n is the total grid squares applied
    // double sumD = 0, sumDsq = 0;
    // for (int w = 0; w < n; w++)
    // {
    //  sumD = sumD + grids[w];
    // sumDsq = sumDsq + (grids[w]*grids[w]);
    // }
    
    //double meandensity = sumD/n;
    //double vardensity = (sumDsq - (sumD*sumD/n))/(n-1);
    //double coVarGrids = Math.sqrt(vardensity)/meandensity;
    // return coVarGrids;
    // }
    
    /**Counts pixels at each box size by calling {@link #count method}
     * for each size.
     * Calls {@link #probmass method} also.  Sums values stored by
     * these calls in {@link #PixPerBox}
     *array and stores average and standard deviation of pixels per box
     *in {@link #StdDev} and {@link #AveragePix} arrays.  Finally,
     *stores the number of boxes that had pixels at this location in a
     *global storage array ({@link #HadPixAllLocAllSize}).
     **Will be rewritten to use FracStats class only.
     * @param wassub boolean true if this was a subarea
     * @param ipc ImageProcessor to work on
     * @param x integer for starting x coordinate
     * @param y integer for startign y coordinate*/
    public void doBoxCounts(boolean wassub, ImageProcessor ipc, int x, int y)
    {
        for (int boxnumber=0; boxnumber<newNumSizes; boxnumber++)
        {/*Goes through for each box size, and counts pixels using that box size*/
            count(wassub, boxnumber, boxSizes[boxnumber], ipc, x, y);
            if (DEBUG)IJ.showMessage("Did count " + Integer.toString(boxnumber));
            
            /*fills PixPerbox global array and grids global array and changes boxes to current count*/
            probmass(boxnumber);
            /*uses global array just stored to make array
             containing how many boxes contained each number of pixels,
             *ie probability of mass for a box size*/
            /*stores a value for the lacunarity using prob
             mass at this box size for this location*/
            AveragePix[boxnumber]=0;
            for (int rr=0; rr<=boxes; rr++)
                /*boxes is set above as current number of boxes counted having pixels*/
                /*calculate average number of pixels per box at this size: sum then divide*/
            {
                AveragePix[boxnumber]=AveragePix[boxnumber]+PixPerBox[rr];
            }
            AveragePix[boxnumber]=AveragePix[boxnumber]/boxes;
            StdDev[boxnumber]=0;
            /*calculate standard deviation of number of pixels per box at this box size*/
            for (int st=0; st <= boxes; st++)
            {
                StdDev[boxnumber] = StdDev[boxnumber]
                + (Math.pow((PixPerBox[st]-AveragePix[boxnumber]), 2));
            }
            
            StdDev[boxnumber] = Math.sqrt(StdDev[boxnumber]/(boxes-1));
            /*is actually variance = stddev^2 so correct now*/
            HadPixAllLocAllSize[locations][boxnumber] = boxes;
        }
        
    }
    
    //////////////////////////////////////////////////////////////////////////
    /** This long-winded method determines the vertices of a polygon which forms a
     * convex hull around the image.  It does this by first mapping the vertical contours
     * of the image.  (Horizontal or vertical contours could have been used, but only one
     * is necessary.)  After they are mapped, all of the points which can be enclosed by
     * connecting any other points are eliminated.  This leaves only the outermost points,
     * the ones that form the convex hull.  Returns a polygon created with these points.
     *<p>written by Thomas Roy, University of Alberta, Canada</P>
     * @param ip ImageProcessor to work on
     * @return ij.gui.PolygonRoi that is convex hull enclosing the image */
    
    public ij.gui.PolygonRoi getPolygon(ImageProcessor ip)
    {
        
        
        int polyXCoordinate[] = new int [maxImageWidth * 2];
        int polyYCoordinate[] = new int [maxImageWidth * 2];
        int[] histogram = new int[256];
        int aa=0;
        int y, x;
        int index = this.left;
        boolean change=true;
        float slope, slope2, rise, run;
        //IJ.showMessage("start left top");
        //Start by mapping farthest left point
        y=top-1;
        do
        {
            y++;
            ip.setRoi(index, y+1, 1, 1);
            histogram = ip.getHistogram();
        } while (( histogram[ this.foreground ]==0 ) && y<=bottom);
        //IJ.showMessage("done left top");
        polyXCoordinate[0]=index;
        polyYCoordinate[0]=y;
        
        int lastx=index;
        int lasty=y;
        float lastslope=0;
        
        //Begin mapping contours left to right
        for (index = left+1; index <= right; index++)
        {
            
            y=top-1;
            
            do
            {
                y++;
                ip.setRoi(index, y+1, 1, 1);
                histogram = ip.getHistogram();
            } while (( histogram[ this.foreground ]==0 ) && y<=bottom);
            
            if (y<=bottom)
            {
                rise = lasty-y;
                run = lastx-index;
                slope = rise/run;
                aa++;
                polyXCoordinate[aa] = index;
                polyYCoordinate[aa]=y;
                lastx=index;
                lasty=y;
                lastslope=slope;
            }
        }
        // IJ.showMessage("done 1");
        //Map farthest right pixel, from top
        y=top-1;//was 1
        index=this.right;
        ip.setRoi(index, y+1, 1, 1);
        histogram = ip.getHistogram();
        if (histogram[this.foreground]==0)
        {
            do
            {
                y++;
                ip.setRoi(index, y+1, 1, 1);
                histogram = ip.getHistogram();
            } while (( histogram[ this.foreground ]==0 ) && y>=top);
        }
        aa++;
        polyXCoordinate[aa]=index+1;
        polyYCoordinate[aa]=y;
        
        int ac=aa;
        
        y=bottom+1;
        index=right;
        // IJ.showMessage("done bot");
        //Map farthest right pixel, from bottom
        do
        {
            y--;
            ip.setRoi(index, y-1, 1, 1);
            histogram = ip.getHistogram();
        } while (( histogram[ this.foreground ]==0 ) && y>=top);
        //IJ.showMessage("done  rb");
        aa++;
        polyXCoordinate[aa]=index+1;
        polyYCoordinate[aa]=y;
        
        lastx=right;
        lasty=y;
        lastslope=0;
        
        //Begin mapping bottom contours, right to left
        for (index = right-1; index >= left; index--)
        {
            
            y=bottom+1;
            
            do
            {
                y--;
                ip.setRoi(index, y-1, 1, 1);
                histogram = ip.getHistogram();
            } while (( histogram[ this.foreground ]==0 ) && y>=top);
            
            if (y>=top)
            {
                rise = lasty-y;
                run = lastx-index;
                slope = rise/run;
                aa++;
                polyXCoordinate[aa] = index;
                polyYCoordinate[aa]=y;
                lastx=index;
                lasty=y;
                lastslope=slope;
                
            }
        }
        //IJ.showMessage("done bb");
        y=bottom+1;
        index=left;
        ////IJ.showMessage("done  ba");
        //Map farthest left pixel, from bottom.  We've completed encircled the pixel area now
        do
        {
            y--;
            ip.setRoi(index, y-1, 1, 1);
            histogram = ip.getHistogram();
        } while (( histogram[ this.foreground ]==0 ) && y>=top);
        //IJ.showMessage("done  ");
        aa++;
        polyXCoordinate[aa]=index-1;
        polyYCoordinate[aa]=y;
        
        // This section of the method removes contours that contain concave
        // areas.  The algorithm tests each contour line to ensure that it has
        // a greater slope than all of the lines after it.  If this is not the case,
        // then it replaces the endpoint with the endpoint of the next contour
        // pixel, and checks again, until it has removed all of the contour points
        // that are concave.
        int ab=0;
        do
        {
            int ad=1;
            do
            {
                rise=polyYCoordinate[ab+1]-polyYCoordinate[ab];
                run=polyXCoordinate[ab+1]-polyXCoordinate[ab];
                if (run==0) run=1;
                slope=rise/run;
                
                rise=polyYCoordinate[ab+ad]-polyYCoordinate[ab];
                run=polyXCoordinate[ab+ad]-polyXCoordinate[ab];
                if (run==0)
                {
                    rise=10000;
                    run=1;
                }
                slope2=rise/run;
                
                if (slope>slope2)
                {
                    polyXCoordinate[ab+1]=polyXCoordinate[ab];
                    polyYCoordinate[ab+1]=polyYCoordinate[ab];
                }
                ad++;
            } while (ad+ab<=ac);
            ab++;
            //IJ.showMessage(new Integer(ab).toString() + " " + new Integer(ad).toString());
            
        } while (ab<ac);
        
        ab=ac+1;
        do
        {
            int ad=2;
            do
            {
                rise=polyYCoordinate[ab+1]-polyYCoordinate[ab];
                run=polyXCoordinate[ab+1]-polyXCoordinate[ab];
                if (run==0) run=1;
                slope=rise/run;
                
                rise=polyYCoordinate[ab+ad]-polyYCoordinate[ab];
                run=polyXCoordinate[ab+ad]-polyXCoordinate[ab];
                if (run==0)
                {
                    run=1;
                    rise=10000;
                }
                slope2=rise/run;
                
                if (slope>slope2)
                {
                    polyXCoordinate[ab+1]=polyXCoordinate[ab];
                    polyYCoordinate[ab+1]=polyYCoordinate[ab];
                }
                ad++;
            } while (ad+ab<=aa);
            ab++;
        } while (ab<=aa);
        
        // remove countour points that are identical copies....
        index=1;
        do
        {
            if (polyXCoordinate[index]==polyXCoordinate[index-1]
            && polyYCoordinate[index]==polyYCoordinate[index-1])
            {
                for(int i=index; i<aa; i++)
                {
                    polyXCoordinate[i]=polyXCoordinate[i+1];
                    polyYCoordinate[i]=polyYCoordinate[i+1];
                }
                if (index<=ac) ac--;
                aa--;
            }	else index++;
        }while (index<=aa);
        
        
        return new ij.gui.PolygonRoi(
        polyXCoordinate,polyYCoordinate, aa+1, img,ij.gui.Roi.POLYGON);
        
    }
    /**Determines the dimensions of the image, and sets some
     * of the important variables. Sets {@link #img} to current window;
     *sets width and height as well as left, right, top,
     * bottom, centrex, and centrey values.
     * @param ip ImageProcessor to work on */
    public void GetDimensions(ImageProcessor ip)
    {
        
        
        img = ij.WindowManager.getCurrentImage();
        int[] histogram = new int[256];/*make an array for histogram*/
        this.width = ip.getWidth();/*record the width and height*/
        this.height = ip.getHeight();
        java.awt.Rectangle roi;
        
        //start at the leftmost edge, move to the middle,
        //and find the first coloured pixel for the left edge
        this.left = -1;
        do
        {	this.left++;
                ip.setRoi(this.left, 0, 1, this.height);
                histogram = ip.getHistogram();
        } while ( histogram[this.foreground]==0 );
        
        //IJ.showMessage("doing l");
        //start at the top and find the first pixel
        this.top = -1;
        do
        {	this.top++;
                ip.setRoi(this.left, this.top, this.width-this.left, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing t");
        //start at the right and find the first pixel
        this.right =this.width-1;
        do
        {	this.right--;
                ip.setRoi(this.right, this.top, 1, this.height-this.top);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing r");
        //start at the bottom of the image and find the first pixel*/
        this.bottom =this.height;
        do
        {	this.bottom--;
                ip.setRoi(this.left, this.bottom, this.right-this.left, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing b");
        // This part of the methods finds the coordinates of the top,
        //bottom, right and left pixels,
        // which is used later for the polygon.
        this.leftyc = 0;
        do
        {	this.leftyc++;
                ip.setRoi(this.left, this.leftyc, 1, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing lyc");
        this.topxc = this.left-1;
        do
        {	topxc++;
                ip.setRoi(topxc, top, 1, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing txc");
        this.rightyc = this.top-1;
        do
        {	this.rightyc++;
                ip.setRoi(this.right, this.rightyc, 1, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing ryc");
        this.bottomxc = this.right;
        do
        {	this.bottomxc--;
                ip.setRoi(this.bottomxc, this.bottom, 1, 1);
                histogram = ip.getHistogram();
        } while (histogram[this.foreground]==0);
        //IJ.showMessage("doing mxc");
        this.imgHeight = bottom - top;
        this.imgWidth = right - left;
        this.centrex = (float)(left + (this.imgWidth/2));
        this.centrey = (float)(top + (this.imgHeight/2));
    }
    
    /************************************************************************/
    /**Gets user inputs from popups to set global variables.  Is
     * invoked by {@link #run} when plugin is
     *first invoked.
     *@see #dosomethreshes
     *@see #incs
     *@see #userNumber
     *@see #maxpercent
     *@see #numlocations
     *@see #doglobal
     *@see #printsummary
     *@see #dosubs
     *@see #fill
     *@see #randomsquares
     *@see #dosubs
     *@see #subboxsize
     *@see #viewplots
     *@see #subsamples
     *@see #docircul
     *@return false if cancelled
     **/
    public boolean getInputs()
    {
        boolean GoOn = true;
        
        GenericDialog gd = new GenericDialog("Automatic threshold", IJ.getInstance());
        gd.addNumericField
        ("1 autothresholds; anything else processes nonbinary images at threshold:"
        , 1, 0);
        gd.showDialog();
        if (gd.wasCanceled())
        {GoOn=false;}else
        {
            if((int) gd.getNextNumber()==1)dosomethreshes=true;
            else dosomethreshes = false;
        }
        if (GoOn)
        {
            gd =
            new GenericDialog("How many boxes per series? 0 for default",
            IJ.getInstance());
            gd.addNumericField("Boxes per series:(0 calculates for you)"
            , 0, 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {GoOn=false;}else
            {
                incs = (int) gd.getNextNumber();
                if (incs<=0)
                {incs = 0;userNumber=false;}
                else userNumber=true;}
        }
        if (GoOn)
        {
            gd
            = new GenericDialog("Maximum percentage of image size to make maximum box size?",
            IJ.getInstance());
            gd.addNumericField("Maximum percent"
            , 47, 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {
                GoOn=false;
            }else
            {
                maxpercent = (double) gd.getNextNumber();
                if (maxpercent<=1)
                {maxpercent = 1;}
            }
        }
        if (GoOn)
        {
            gd = new GenericDialog("Number of origins for global analysis", IJ.getInstance());
            gd.addNumericField("How many global origins?:", numlocations, 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {
                GoOn=false;
            }else
            {
                numlocations = (int) gd.getNextNumber();
                if (numlocations <= 0)
                {numlocations = 1;doglobal=false;}
            }}
        
        if (false)
        {
            gd =  new GenericDialog("DEBUG?", IJ.getInstance());
            gd.addStringField("(y)es or (n)o:", "N", 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {GoOn=false;}
            else
            {
                String d = gd.getNextString();
                if (d.toLowerCase().startsWith("y")) DEBUG=true;
                else DEBUG=false;
            }
        }
        if (GoOn)
        {
            gd =  new GenericDialog("Summary Only?", IJ.getInstance());
            gd.addStringField("Summarize: (y)es or (n)o:", "Y", 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {GoOn=false;}
            else
            {printsummary = gd.getNextString();
             if (printsummary.toLowerCase().startsWith("y")) printsummary = "y";
             else printsummary = "n";
            }}
        if (GoOn)
        {
            dosubs=false;
            gd =  new GenericDialog("Do Subareas?", IJ.getInstance());
            gd.addStringField("(Y)es or (N)o:", "N", 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {GoOn=false;}
            else
            {
                String su = gd.getNextString();
                if (su.toLowerCase().startsWith("y")) dosubs=true;
                
            }}
        if (GoOn)
        {
            if (dosubs==true)
            {
                gd = new GenericDialog("Size of boxes (pixels)", IJ.getInstance());
                gd.addNumericField("Size for box (corrected if invalid)", 30, 0);
                gd.showDialog();
                if (gd.wasCanceled())
                {GoOn=false;}
                else
                {  subboxsize = (int) gd.getNextNumber();
                   if (subboxsize<=0)subboxsize=2;}
                
                if (GoOn)
                {
                    gd =  new GenericDialog("See colours?", IJ.getInstance());
                    gd.addStringField("(y)es or (n)o:", "Y", 0);
                    gd.showDialog();
                    if (gd.wasCanceled())
                    {GoOn=false;}
                    else
                    {
                        String fd = gd.getNextString();
                        if (fd.toLowerCase().startsWith("y"))
                        { showcolours=true;}
                        else showcolours = false;
                    }
                }
                if (GoOn)
                {
                    if (showcolours)
                    {
                        String filled = "Y";
                        gd =  new GenericDialog("Fill squares?", IJ.getInstance());
                        gd.addStringField("(y)es or (n)o:", filled, 0);
                        gd.showDialog();
                        if (gd.wasCanceled())
                        {GoOn=false;}
                        else
                        {
                            filled = gd.getNextString();
                            if (filled.toLowerCase().startsWith("n"))
                            { filled = "n";fill=false;}
                            else fill = true;
                        }
                    }}
                if (GoOn)
                {
                    String rand = "n";
                    gd =  new GenericDialog("Random squares?", IJ.getInstance());
                    gd.addStringField("(y)es or (n)o:", rand, 0);
                    gd.showDialog();
                    if (gd.wasCanceled())
                    {GoOn=false;}
                    else
                    {
                        rand = gd.getNextString();
                        if (rand.toLowerCase().startsWith("y"))
                        { randomsquares=true;}
                        else randomsquares = false;
                    }}
                if (GoOn)
                {
                    if (randomsquares)
                    {
                        gd = new GenericDialog("How many areas?", IJ.getInstance());
                        gd.addNumericField("How many subsamples?:", 20, 0);
                        gd.showDialog();
                        if (gd.wasCanceled())
                        {GoOn=false;}
                        else
                        {
                            subsamples = (int) gd.getNextNumber();
                            if (subsamples<=0) subsamples=1;
                        }
                    }
                }
            }}
        
        if (GoOn&&(dosubs==false))
        {
            gd = new GenericDialog("View plots?", IJ.getInstance());
            gd.addStringField("See regression lines?:", "Y", 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {GoOn=false;}
            else
            {
                String ss = gd.getNextString();
                if (ss.toLowerCase().startsWith("y")) viewplots=true;
                else viewplots=false;
            }
        }
        
        if(GoOn)
        {
            
            gd = new GenericDialog("Find circularity", IJ.getInstance());
            gd.addNumericField("1 to include circularity data", 1, 0);
            gd.showDialog();
            if (gd.wasCanceled())
            {
                GoOn=false;
            }
            else
            {docircul = (int) gd.getNextNumber();
            }}
        return GoOn;
        
    }
    /************************************************************************/
    /**Lists the headings for the results file.</p>
     *  "FracLac2003 FILE" <p></p>
     * "AVG Box Count Df" <p></p>
     * "r squared" <p></p>
     * "Corrected"<p></p>
     * "r squared" <p></p>
     * "Coef Var Lacunarity"<p></p>
     * "Avg Coef Var for Y-Int at Multiple Origins"<p></p>
     * "Probability Density Lacunarity
     * (Coef Var over all epsilon at Multiple Locations)" <p></p>
     * "HULL AREA" <p></p>
     * "HULL PERIMETER"<p></p>
     * "ROUNDNESS"<p></p>
     * "CIRCULARITY"<p></p>
     * "WIDTH"<p></p>
     * "HEIGHT"<p></p>
     * "VERTICAL"<p></p>
     * "HORIZONTAL"<p></p>
     * "CENTRE (origin)"<p></p>
     * "NUM BOXES"<p></p>
     * "adjusted NUM BOXES"<p></p>
     * "Minimum Cover Df"<p></p>
     * "r squared"<p></p>
     * "Corrected" <p></p>
     * "r squared" <p></p>
     * "Coef Var for all Y-int"<p></p>
     * "Coef Variation Df" <p></p>
     * "Number of origins" <p></p>
     * @return string of headings.*/
    public String makeheadings()
    {
        String headings =
        "FracLac2003 FILE" + "\t"+
        "AVG Box Count Df" + tab +
        "r squared" + tab+
        "Corrected"+ tab+
        "r squared" + tab+
        "Coef Var Lacunarity"+ "\t"+
        "Avg Coef Var for Y-Int at Multiple Origins"+ "\t"+
        "Probability Density Lacunarity (Coef Var over all epsilon at Multiple Locations)" +"\t"+
        "HULL AREA" +"\t"+
        "HULL PERIMETER"+ "\t"+
        "ROUNDNESS"+ "\t"+
        "CIRCULARITY"+ "\t"+
        "WIDTH"+ "\t" +
        "HEIGHT" + "\t"+
        "VERTICAL"   + "\t"+
        "HORIZONTAL" + "\t"+
        "CENTRE (origin)" + "\t "+
        "NUM BOXES"+ "\t "+
        "adjusted NUM BOXES"+ "\t "+
        "Minimum Cover Df" + tab +
        "r squared" + tab+
        "Corrected" + tab +
        "r squared" + tab+
        "Coef Var for all Y-int"  + "\t"        +
        "Coef Variation Df" + "\t"    +
        "Number of origins" ;
        return headings;
        
    }
    
    /************************************************************************/
    /** Reinitializes {@link #PixPerBox} array
     * then goes over part of the ip using
     * a series of smaller boxes recording number of pixels in
     * each box and number of boxes that had pixels. Is called by
     * {@link #count method}. Changes variables for actualx and
     * actualy each time.
     *
     * @param ystart integer for y coordinate to start at
     * @param sz integer for size of box
     * @param bot integer for bottom edge limit
     * @param rt integer for right edge limit
     * @param boxnum integer for later implementation of grid density
     * @param xstart integer for starting x coordinate
     * @param ipr ImageProcessor to analyze
     * @return countboxes int for number of boxes that had pixels*/
    public int moveGridalloverCountPixels(
    int boxnum,
    int xstart,
    int ystart,
    int sz,
    int bot,
    int rt,
    ImageProcessor ipr)
    
    {
        int countboxes=0;
        boolean done = false;
        int numPix = 0;
        if (DEBUG)IJ.showMessage("Making arrays");
        PixPerBox = new int[stats.pixelCount];
        if (DEBUG)IJ.showMessage("Made pixperbox array");
        /**grids = new double[stats.pixelCount];*/
        //IJ.showMessage("Made grids array");
        for (int y =ystart; y <=bot; y=y+sz)
        {
            for ( int x = xstart;x<=rt; x=x+sz)
            {
                ipr.setRoi(x,y,sz,sz);
                histogram = ip.getHistogram();
                numPix = histogram[foreground];
                /*grids[boxnum] = numPix/(size*size);*/
                if (numPix>0)
                {
                    countboxes++;/*count if there were pixels in the box*/
                    PixPerBox[countboxes-1]=numPix;
                }
            }
        }
        actualx = xstart;/*set these for printing the box locations*/
        actualy = ystart;/*set these for printing the box locations*/
        return countboxes;
        /*returns the number of boxes that had pixels*/
    }
    
    /************************************************************************/
    
}/*end plugin*/
